require 'twitter'

Twitter::Tweet.class_eval do
  def is_pure_user_tweet?(username)
    self.in_reply_to_screen_name.nil? &&\
      !self.full_text.include?("@#{username}")
  end
end

module Ruboty
  module Tweetstream
    class Stream
      attr_reader :hash

      def initialize(hash)
        @hash = hash
        @thread = nil

        @client = Twitter::Streaming::Client.new do |config|
          config.consumer_key        = CONSUMER_KEY
          config.consumer_secret     = CONSUMER_SECRET
          config.access_token        = ACCESS_TOKEN
          config.access_token_secret = ACCESS_TOKEN_SECRET
        end
      end

      def start(robot)
        @thread = interval_reconnect do
          @client.filter(follow: hash[:twitter_id].to_s) do |object|
            if object.is_a?(Twitter::Tweet) && object.is_pure_user_tweet?(hash[:username])
              Message.new(
                hash.except(:username, :twitter_id).merge(robot: robot)
              ).reply("@#{hash[:username]}> #{object.full_text}")
            end
          end
        end
      end

      # Twitter::Streaming disconnects over time
      def interval_reconnect
        parent = Thread.start do
          loop do
            child = Thread.start do
              yield
            end
            sleep 3600 # Disconnect every hour
            child.kill
          end
        end

        parent
      end

      def stop
        @thread.kill
      end
    end
  end
end
